@@ -11,14 +11,19 @@ import time
 import warnings
 import sys
 #site
+try:
+    from pkg_resources import resource_filename
+except ImportError:
+    resource_filename = None
 #pkg
 from passlib import hash
 from passlib.context import CryptContext, CryptPolicy, LazyCryptContext
-from passlib.utils import to_bytes
+from passlib.utils import to_bytes, to_unicode
 import passlib.utils.handlers as uh
-from passlib.tests.utils import TestCase, mktemp, catch_warnings
+from passlib.tests.utils import TestCase, mktemp, catch_warnings, \
+    gae_env, set_file
 from passlib.registry import register_crypt_handler_path, has_crypt_handler, \
-        _unload_handler_name as unload_handler_name
+    _unload_handler_name as unload_handler_name
 #module
 log = getLogger(__name__)
 
@@ -39,6 +44,7 @@ class CryptPolicyTest(TestCase):
     #-----------------------------------------------------
     #sample 1 - average config file
     #-----------------------------------------------------
+    #NOTE: copy of this is stored in file passlib/tests/sample_config_1s.cfg
     sample_config_1s = """\
 [passlib]
 schemes = des_crypt, md5_crypt, bsdi_crypt, sha512_crypt
@@ -49,6 +55,15 @@ bsdi_crypt.default_rounds = 25000
 sha512_crypt.max_rounds = 50000
 sha512_crypt.min_rounds = 40000
 """
+    sample_config_1s_path = os.path.abspath(os.path.join(
+        os.path.dirname(__file__), "sample_config_1s.cfg"))
+    if not os.path.exists(sample_config_1s_path) and resource_filename:
+        #in case we're zipped up in an egg.
+        sample_config_1s_path = resource_filename("passlib.tests",
+                                                  "sample_config_1s.cfg")
+
+    #make sure sample_config_1s uses \n linesep - tests rely on this
+    assert sample_config_1s.startswith("[passlib]\nschemes")
 
     sample_config_1pd = dict(
         schemes = [ "des_crypt", "md5_crypt", "bsdi_crypt", "sha512_crypt"],
@@ -181,46 +196,70 @@ admin.sha512_crypt.max_rounds = 40000
                           schemes=['des_crypt'],
                           default='md5_crypt')
 
-    def test_01_from_path(self):
+    def test_01_from_path_simple(self):
         "test CryptPolicy.from_path() constructor"
+        #NOTE: this is separate so it can also run under GAE
+
+        #test preset stored in existing file
+        path = self.sample_config_1s_path
+        policy = CryptPolicy.from_path(path)
+        self.assertEqual(policy.to_dict(), self.sample_config_1pd)
+
+        #test if path missing
+        self.assertRaises(EnvironmentError, CryptPolicy.from_path, path + 'xxx')
+
+    def test_01_from_path(self):
+        "test CryptPolicy.from_path() constructor with encodings"
+        if gae_env:
+            return self.skipTest("GAE doesn't offer read/write filesystem access")
+
         path = mktemp()
-        with open(path, "w") as fh:
-            fh.write(self.sample_config_1s)
+
+        #test "\n" linesep
+        set_file(path, self.sample_config_1s)
+        policy = CryptPolicy.from_path(path)
+        self.assertEqual(policy.to_dict(), self.sample_config_1pd)
+
+        #test "\r\n" linesep
+        set_file(path, self.sample_config_1s.replace("\n","\r\n"))
         policy = CryptPolicy.from_path(path)
         self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
         #test with custom encoding
         uc2 = to_bytes(self.sample_config_1s, "utf-16", source_encoding="utf-8")
-        with open(path, "wb") as fh:
-            fh.write(uc2)
+        set_file(path, uc2)
         policy = CryptPolicy.from_path(path, encoding="utf-16")
         self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
-        #test if path missing
-        os.remove(path)
-        self.assertRaises(EnvironmentError, CryptPolicy.from_path, path)
-
     def test_02_from_string(self):
         "test CryptPolicy.from_string() constructor"
+        #test "\n" linesep
         policy = CryptPolicy.from_string(self.sample_config_1s)
         self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
-        policy = CryptPolicy.from_string(self.sample_config_4s)
-        self.assertEqual(policy.to_dict(), self.sample_config_4pd)
+        #test "\r\n" linesep
+        policy = CryptPolicy.from_string(
+            self.sample_config_1s.replace("\n","\r\n"))
+        self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
-        #test with custom encoding
+        #test with unicode
+        data = to_unicode(self.sample_config_1s)
+        policy = CryptPolicy.from_string(data)
+        self.assertEqual(policy.to_dict(), self.sample_config_1pd)
+
+        #test with non-ascii-compatible encoding
         uc2 = to_bytes(self.sample_config_1s, "utf-16", source_encoding="utf-8")
         policy = CryptPolicy.from_string(uc2, encoding="utf-16")
         self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
+        #test category specific options
+        policy = CryptPolicy.from_string(self.sample_config_4s)
+        self.assertEqual(policy.to_dict(), self.sample_config_4pd)
+
     def test_03_from_source(self):
         "test CryptPolicy.from_source() constructor"
-
         #pass it a path
-        path = mktemp()
-        with open(path, "w") as fh:
-            fh.write(self.sample_config_1s)
-        policy = CryptPolicy.from_source(path)
+        policy = CryptPolicy.from_source(self.sample_config_1s_path)
         self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
         #pass it a string
@@ -250,11 +289,9 @@ admin.sha512_crypt.max_rounds = 40000
         self.assertEqual(policy.to_dict(), self.sample_config_1pd)
 
         #pass multiple sources
-        path = mktemp()
-        with open(path, "w") as fh:
-            fh.write(self.sample_config_1s)
-        policy = CryptPolicy.from_sources([
-            path,
+        policy = CryptPolicy.from_sources(
+            [
+            self.sample_config_1s_path,
             self.sample_config_2s,
             self.sample_config_3pd,
             ])
@@ -719,21 +756,54 @@ class CryptContextTest(TestCase):
             self.assertTrue(not cc.verify("test", None, scheme=handler.name))
 
     def test_24_min_verify_time(self):
-        cc = CryptContext(["plaintext", "bsdi_crypt"], min_verify_time=.1)
+        "test verify() honors min_verify_time"
+        #NOTE: this whole test assumes time.sleep() and time.time()
+        #      have at least 1ms accuracy
+        delta = .1
+        min_delay = delta
+        min_verify_time = min_delay + 2*delta
+        max_delay = min_verify_time + 2*delta
 
-        #plaintext should (in reality) take <.01,
-        #so this test checks mvt makes it take 0.09 - .5
-        s = time.time()
-        cc.verify("password", "password")
-        d = time.time()-s
-        self.assertTrue(d>=.09,d)
-        self.assertTrue(d<.5)
+        class TimedHash(uh.StaticHandler):
+            "psuedo hash that takes specified amount of time"
+            name = "timed_hash"
+            delay = 0
 
-        #this may take longer, so we just check min
-        s = time.time()
-        cc.verify("password", '_2b..iHVSUNMkJT.GcFU')
-        d = time.time()-s
-        self.assertTrue(d>=.09, "mvt=.1, delta=%r" % (d,))
+            @classmethod
+            def identify(cls, hash):
+                return True
+
+            @classmethod
+            def genhash(cls, secret, hash):
+                time.sleep(cls.delay)
+                return hash or 'x'
+
+        cc = CryptContext([TimedHash], min_verify_time=min_verify_time)
+
+        def timecall(func, *args, **kwds):
+            start = time.time()
+            result = func(*args, **kwds)
+            end = time.time()
+            return end-start, result
+
+        #verify hashing works
+        TimedHash.delay = min_delay
+        elapsed, _ = timecall(TimedHash.genhash, 'stub', 'stub')
+        self.assertAlmostEqual(elapsed, min_delay, delta=delta)
+
+        #ensure min verify time is honored
+        elapsed, _ = timecall(cc.verify, "stub", "stub")
+        self.assertAlmostEqual(elapsed, min_verify_time, delta=delta)
+
+        #ensure taking longer emits a warning.
+        TimedHash.delay = max_delay
+        with catch_warnings(record=True) as wlog:
+            warnings.simplefilter("always")
+            elapsed, _ = timecall(cc.verify, "stub", "stub")
+        self.assertAlmostEqual(elapsed, max_delay, delta=delta)
+        self.assertEqual(len(wlog), 1)
+        self.assertWarningMatches(wlog[0],
+            message_re="CryptContext: verify exceeded min_verify_time")
 
     def test_25_verify_and_update(self):
         "test verify_and_update()"
