@@ -2,6 +2,7 @@
 #=========================================================
 #imports
 #=========================================================
+from __future__ import with_statement
 #core
 import atexit
 import logging; log = logging.getLogger(__name__)
@@ -27,17 +28,6 @@ except ImportError:
 import warnings
 from warnings import warn
 
-try:
-    from warnings import catch_warnings
-except ImportError:
-    #catch_warnings wasn't added until py26
-    #put stub in place
-    class catch_warnings(object):
-        def __enter__(self):
-            return None
-        def __exit__(self, *exc_info):
-            return None
-        __name__ = 'stub'
 #site
 if ut_version < 2:
     #used to provide replacement skipTest() method
@@ -52,14 +42,28 @@ __all__ = [
     #util funcs
     'enable_option',
     'Params',
+    'set_file', 'get_file',
 
     #unit testing
     'TestCase',
     'HandlerCase',
     'enable_backend_case',
     'create_backend_case',
+
+    #flags
+    'gae_env',
 ]
 
+#figure out if we're running under GAE...
+#some tests (eg FS related) should be skipped.
+    #XXX: is there better way to do this?
+try:
+    import google.appengine
+except ImportError:
+    gae_env = False
+else:
+    gae_env = True
+
 #=========================================================
 #option flags
 #=========================================================
@@ -111,6 +115,18 @@ class Params(object):
             txt = txt[:-2]
         return txt
 
+def set_file(path, content):
+    "set file to specified bytes"
+    if isinstance(content, unicode):
+        content = content.encode("utf-8")
+    with open(path, "wb") as fh:
+        fh.write(content)
+
+def get_file(path):
+    "read file as bytes"
+    with open(path, "rb") as fh:
+        return fh.read()
+
 #=========================================================
 #custom test base
 #=========================================================
@@ -214,7 +230,7 @@ class TestCase(unittest.TestCase):
         raise self.failureException(msg)
 
     #===============================================================
-    #add some extra methods (these are already present in unittest2)
+    #backport some methods from unittest2
     #===============================================================
     if ut_version < 2:
 
@@ -239,6 +255,56 @@ class TestCase(unittest.TestCase):
         def skipTest(self, reason):
             raise SkipTest(reason)
 
+        def assertAlmostEqual(self, first, second, places=None, msg=None, delta=None):
+            """Fail if the two objects are unequal as determined by their
+               difference rounded to the given number of decimal places
+               (default 7) and comparing to zero, or by comparing that the
+               between the two objects is more than the given delta.
+
+               Note that decimal places (from zero) are usually not the same
+               as significant digits (measured from the most signficant digit).
+
+               If the two objects compare equal then they will automatically
+               compare almost equal.
+            """
+            if first == second:
+                # shortcut
+                return
+            if delta is not None and places is not None:
+                raise TypeError("specify delta or places not both")
+
+            if delta is not None:
+                if abs(first - second) <= delta:
+                    return
+
+                standardMsg = '%s != %s within %s delta' % (repr(first),
+                                                            repr(second),
+                                                            repr(delta))
+            else:
+                if places is None:
+                    places = 7
+
+                if round(abs(second-first), places) == 0:
+                    return
+
+                standardMsg = '%s != %s within %r places' % (repr(first),
+                                                              repr(second),
+                                                              places)
+            msg = self._formatMessage(msg, standardMsg)
+            raise self.failureException(msg)
+
+    if not hasattr(unittest.TestCase, "assertRegexpMatches"):
+        #added in 2.7/UT2 and 3.1        
+        def assertRegexpMatches(self, text, expected_regex, msg=None):
+            """Fail the test unless the text matches the regular expression."""
+            if isinstance(expected_regex, basestring):
+                assert expected_regex, "expected_regex must not be empty."
+                expected_regex = re.compile(expected_regex)
+            if not expected_regex.search(text):
+                msg = msg or "Regex didn't match"
+                msg = '%s: %r not found in %r' % (msg, expected_regex.pattern, text)
+                raise self.failureException(msg)
+
     #============================================================
     #add some custom methods
     #============================================================
@@ -257,6 +323,50 @@ class TestCase(unittest.TestCase):
             msg = "error for case %r:" % (elem.render(1),)
             self.assertEqual(result, correct, msg)
 
+    def assertWarningMatches(self, warning,
+                             message=None, message_re=None,
+                             category=None,
+                             ##filename=None, filename_re=None,
+                             ##lineno=None,
+                             msg=None,
+                             ):
+        "check if WarningMessage instance (as returned by catch_warnings) matches parameters"
+
+        #determine if we have WarningMessage object,
+        #and ensure 'warning' contains only warning instances.
+        if hasattr(warning, "category"):
+            wmsg = warning
+            warning = warning.message
+        else:
+            wmsg = None
+
+        #tests that can use a warning instance or WarningMessage object
+        if message:
+            self.assertEqual(str(warning), message, msg)
+        if message_re:
+            self.assertRegexpMatches(str(warning), message_re, msg)
+        if category:
+            self.assertIsInstance(warning, category, msg)
+
+        #commented out until needed...
+        ###tests that require a WarningMessage object
+        ##if filename or filename_re:
+        ##    if not wmsg:
+        ##        raise TypeError("can't read filename from warning object")
+        ##    real = wmsg.filename
+        ##    if real.endswith(".pyc") or real.endswith(".pyo"):
+        ##        #FIXME: should use a stdlib call to resolve this back
+        ##        #       to original module's path
+        ##        real = real[:-1]
+        ##    if filename:
+        ##        self.assertEqual(real, filename, msg)
+        ##    if filename_re:
+        ##        self.assertRegexpMatches(real, filename_re, msg)
+        ##if lineno:
+        ##    if not wmsg:
+        ##        raise TypeError("can't read lineno from warning object")
+        ##    self.assertEqual(wmsg.lineno, lineno, msg)
+
     #============================================================
     #eoc
     #============================================================
@@ -907,7 +1017,7 @@ def _has_other_backends(handler, ignore):
             return name
     return None
 
-def create_backend_case(base, name):
+def create_backend_case(base, name, module="passlib.tests.test_drivers"):
     "create a test case for specific backend of a multi-backend handler"
     #get handler, figure out if backend should be tested
     handler = base.handler
@@ -919,13 +1029,17 @@ def create_backend_case(base, name):
     if not enable and ut_version < 2:
         return None
 
+    #make classname match what it's stored under, to be tidy
+    cname = name.title().replace("_","") + "_" + base.__name__.lstrip("_")
+
     #create subclass of 'base' which uses correct backend
     subcase = type(
-        "%s_%s" % (name.title(), base.__name__),
+        cname,
         (base,),
         dict(
             case_prefix = "%s (%s backend)" % (handler.name, name),
             backend = name,
+            __module__=module,
         )
     )
 
@@ -972,6 +1086,96 @@ def mktemp(*args, **kwds):
     os.close(fd)
     return path
 
+#=========================================================
+#make sure catch_warnings() is available
+#=========================================================
+try:
+    from warnings import catch_warnings
+except ImportError:
+    #catch_warnings wasn't added until py26.
+    #this adds backported copy from py26's stdlib
+    #so we can use it under py25.
+
+    class WarningMessage(object):
+
+        """Holds the result of a single showwarning() call."""
+
+        _WARNING_DETAILS = ("message", "category", "filename", "lineno", "file",
+                            "line")
+
+        def __init__(self, message, category, filename, lineno, file=None,
+                        line=None):
+            local_values = locals()
+            for attr in self._WARNING_DETAILS:
+                setattr(self, attr, local_values[attr])
+            self._category_name = category.__name__ if category else None
+
+        def __str__(self):
+            return ("{message : %r, category : %r, filename : %r, lineno : %s, "
+                        "line : %r}" % (self.message, self._category_name,
+                                        self.filename, self.lineno, self.line))
+
+
+    class catch_warnings(object):
+
+        """A context manager that copies and restores the warnings filter upon
+        exiting the context.
+
+        The 'record' argument specifies whether warnings should be captured by a
+        custom implementation of warnings.showwarning() and be appended to a list
+        returned by the context manager. Otherwise None is returned by the context
+        manager. The objects appended to the list are arguments whose attributes
+        mirror the arguments to showwarning().
+
+        The 'module' argument is to specify an alternative module to the module
+        named 'warnings' and imported under that name. This argument is only useful
+        when testing the warnings module itself.
+
+        """
+
+        def __init__(self, record=False, module=None):
+            """Specify whether to record warnings and if an alternative module
+            should be used other than sys.modules['warnings'].
+
+            For compatibility with Python 3.0, please consider all arguments to be
+            keyword-only.
+
+            """
+            self._record = record
+            self._module = sys.modules['warnings'] if module is None else module
+            self._entered = False
+
+        def __repr__(self):
+            args = []
+            if self._record:
+                args.append("record=True")
+            if self._module is not sys.modules['warnings']:
+                args.append("module=%r" % self._module)
+            name = type(self).__name__
+            return "%s(%s)" % (name, ", ".join(args))
+
+        def __enter__(self):
+            if self._entered:
+                raise RuntimeError("Cannot enter %r twice" % self)
+            self._entered = True
+            self._filters = self._module.filters
+            self._module.filters = self._filters[:]
+            self._showwarning = self._module.showwarning
+            if self._record:
+                log = []
+                def showwarning(*args, **kwargs):
+                    log.append(WarningMessage(*args, **kwargs))
+                self._module.showwarning = showwarning
+                return log
+            else:
+                return None
+
+        def __exit__(self, *exc_info):
+            if not self._entered:
+                raise RuntimeError("Cannot exit %r without entering first" % self)
+            self._module.filters = self._filters
+            self._module.showwarning = self._showwarning
+
 #=========================================================
 #EOF
 #=========================================================
